<?php
/**
 * Plugin Name: Siuuuu Sitemap Generator
 * Plugin URI: https://siuuuu.cc/
 * Description: Custom sitemap generator plugin to fix content-type and indexability issues with nginx and SEO plugins.
 * Version: 1.0
 * Author: Siuuuu
 * Author URI: https://siuuuu.cc/
 */

add_action('init', function () {
    add_rewrite_rule('^sitemap_index.xml$', 'index.php?siuuuu_sitemap=index', 'top');
    add_rewrite_rule('^post-sitemap.xml$', 'index.php?siuuuu_sitemap=post', 'top');
    add_rewrite_rule('^page-sitemap.xml$', 'index.php?siuuuu_sitemap=page', 'top');
    add_rewrite_rule('^category-sitemap.xml$', 'index.php?siuuuu_sitemap=category', 'top');
    add_rewrite_rule('^post_tag-sitemap.xml$', 'index.php?siuuuu_sitemap=post_tag', 'top');
    add_rewrite_rule('^author-sitemap.xml$', 'index.php?siuuuu_sitemap=author', 'top');
    flush_rewrite_rules();
});

add_filter('query_vars', function ($vars) {
    $vars[] = 'siuuuu_sitemap';
    return $vars;
});

add_action('template_redirect', function () {
    $type = get_query_var('siuuuu_sitemap');
    if ($type) {
        header('Content-Type: application/xml; charset=UTF-8');
        header('X-Robots-Tag: index, follow');
        echo '<?xml version="1.0" encoding="UTF-8"?>';

        $home = home_url();

        if ($type === 'index') {
            echo "<sitemapindex xmlns='http://www.sitemaps.org/schemas/sitemap/0.9'>";
            foreach (['post', 'page', 'category', 'post_tag', 'author'] as $section) {
                echo "<sitemap><loc>{$home}/{$section}-sitemap.xml</loc><lastmod>" . gmdate('c') . "</lastmod></sitemap>";
            }
            echo "</sitemapindex>";
        } else {
            echo "<urlset xmlns='http://www.sitemaps.org/schemas/sitemap/0.9'>";
            $query_args = ['post_type' => $type === 'post_tag' ? 'post' : $type, 'posts_per_page' => -1];
            if ($type === 'category' || $type === 'post_tag' || $type === 'author') {
                if ($type === 'category') $terms = get_categories();
                if ($type === 'post_tag') $terms = get_tags();
                if ($type === 'author') $terms = get_users(['who' => 'authors']);
                foreach ($terms as $term) {
                    $url = get_term_link($term);
                    echo "<url><loc>{$url}</loc><lastmod>" . gmdate('c') . "</lastmod></url>";
                }
            } else {
                $posts = get_posts($query_args);
                foreach ($posts as $post) {
                    $url = get_permalink($post);
                    $modified = get_post_modified_time('c', true, $post);
                    echo "<url><loc>{$url}</loc><lastmod>{$modified}</lastmod></url>";
                }
            }
            echo "</urlset>";
        }
        exit;
    }
});
